/*
 Volume Rendering Demo
 
 MIT License
 
 Copyright (c) 2011, Markus Broecker <mbroecker@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */

#include "OrbitCamera.h"
#include <OpenGL/OpenGL.h>
#include <GLUT/GLUT.h>
#include <cmath>
#include <algorithm>

OrbitCamera::OrbitCamera() : position(0, 0, -1), up(0, 1, 0), radius(5.f), fov(60.f), ratio(1.3f), near(1.f), far(1000.f)
{
}

static inline float radians(float deg)
{
	return deg*0.017453292519943;
}

void OrbitCamera::rotateTo(float x, float y)
{
	y = std::min(1.f, std::max(-1.f, y));
	theta = ratio*178*y;
	phi = x*360.f;
	
	updatePosition();
}

void OrbitCamera::rotate(float x, float y)
{
	theta = std::min(89.f, std::max(-89.f, theta+y));
	phi += x;
	
	updatePosition();
}

void OrbitCamera::updatePosition()
{
	float T = radians(theta);
	float P = radians(phi);
	
	position.x = sin(T)*cos(P)*radius;
	position.y = cos(T)*radius;
	position.z = sin(T)*sin(P)*radius;	
}

void OrbitCamera::setup() const
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(fov, ratio, near, far);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	Vec3 eye = target + position;
	gluLookAt(eye.x, eye.y, eye.z, target.x, target.y, target.z, up.x, up.y, up.z);
	
	
}

