/*
	GLSL Shader abstraction
 
	MIT License

	Copyright (c) 2009, Markus Broecker <mbroecker@gmail.com>

	Permission is hereby granted, free of charge, to any person obtaining a copy
	of this software and associated documentation files (the "Software"), to deal
	in the Software without restriction, including without limitation the rights
	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	copies of the Software, and to permit persons to whom the Software is
	furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
	THE SOFTWARE.
*/

#ifndef SHADER_INCLUDED
#define SHADER_INCLUDED

#include <string>
#include <OpenGL/OpenGL.h>

	
class Shader
{
public:
	/// \name Destruction
	/// \{
	Shader(const std::string& vpFile, const std::string& fpFile);
	~Shader();
	/// \}

	/// \name Interface
	/// \{
	void bind() const;
	void disable() const;

	int getUniform(const std::string& name) const;

	void setUniform(const std::string& name, int i) const;
	void setUniform(const std::string& name, float f) const;
	void setUniform(const std::string& name, float x, float y) const;
	void setUniform(const std::string& name, float x, float y, float z) const;
	void setUniform(const std::string& name, float x, float y, float z, float w) const;
	void setMatrix4(const std::string& name, const float* matrix, bool transpose=false) const;
	void setMatrix4(const std::string& name, const double* matrix, bool transpose=false) const;
	void setTexture1D(const std::string& textureName, unsigned int textureId,unsigned int textureUnit=0) const;
	void setTexture2D(const std::string& textureName, unsigned int textureId,unsigned int textureUnit=0) const;
	void setTexture3D(const std::string& textureName, unsigned int textureId,unsigned int textureUnit=0) const;
	void setTextureCube(const std::string& textureName, unsigned int textureId, unsigned int textureUnit=0) const;

	/*
	virtual void setTexture2D(const std::string& textureName, const Texture& tex, unsigned int textureUnit=0) const = 0;
	virtual void setTextureCube(const std::string& textureName, const CubeMapTexture& cubemap, unsigned int textureUnit=0) const = 0;
	*/
	
	/// \}
	
private:
	Shader(const Shader& sh);
	
	int				mVS, mFS, mProgram;		

	void load(const std::string& vertexFile, const std::string& fragmentFile);
	void loadShader(const std::string& fileName, GLenum type);
};
	

#endif
