/*
 Volume Rendering Demo
 
 MIT License
 
 Copyright (c) 2011, Markus Broecker <mbroecker@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */

#ifndef VOLMETDATA_INCLUDED
#define VOLMETDATA_INCLUDED

#include <string>
#include <vector>


/**	VolumetricData holds and loads volumetric data from files. It must be 
	overridden with specific rendering methods!
*/
class VolumetricData
{
public:
	VolumetricData();
	virtual ~VolumetricData();
	
	/// loads unsigned char raw data files
	void loadRAW(const std::string& filename,  unsigned int width, unsigned int height, unsigned int depth);
		
	/// Gets called after data has been loaded.
	virtual void updatedData() = 0;
	
	/// Draws this volume
	virtual void draw() const = 0;
	
	
protected:
	typedef	char DensityType;
	
	DensityType*		data;
	unsigned int		width, height, depth;
};

class SlicedVolume : public VolumetricData
{
public:
	~SlicedVolume();
	
	/// Updates texture data
	void updatedData();
	void draw() const;
	
	
private:
	// the number of slices is the same as the 'depth'
	std::vector<unsigned int>	textures;
};


#endif
