/*
 Volume Rendering Demo
 
 MIT License
 
 Copyright (c) 2011, Markus Broecker <mbroecker@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */


#version 120

uniform sampler2D	slice;
uniform float		threshold=0.2;
uniform	vec3		low_density = vec3(0.2, 0.7, 0.2);
uniform	vec3		high_density = vec3(0.7, 0.0, 0.2);
uniform float		transparency = 0.0;

varying vec2		texcoord;


void main()
{
	float density = texture2D(slice, vec2(1.0)-texcoord).x;
	//float density = texture2D(slice, texcoord).x;
	
	
	if (density < threshold)
		discard;
	
	vec3 color = mix(low_density, high_density, density);
	
	if (transparency < 0.1)
		density = 1.0;
	
	gl_FragColor = vec4(color, density);
}

